{
    File      : Sprites.Pas
    Author    : Jay Giganti
    Company   : Word in Action
                Copyright (C) 1995 by Jay Giganti.  All Rights Reserved.

    Overview  : This unit implements a simple sprite Class  I did some
                Timings on my machine with HANG_ON defined in sprite_s.pas
                and only one ship flying I Found that BitBlts I got about 154
                Draws/Sec and with CopyRect I got 408 Draws/Sec.  It had to due
                with the fact that I needed to Invalidate a portion of the
                canvas so that it would be redrawn.  When I changed the one
                BitBlt to copy from the offscreen bitmap to Forms canvas then I
                got about 399 Draws/Sec with BitBlt So I am sticking with
                CopyRect.
                
                
    Disclaimer: This code is for demonstration purposes only and should
                be used at your own risk

                  Any comments, suggestions or improvements:
                  Jay Giganti -- CompuServe ID: [72037,2225]


                              Version Information
      Date    Ver  Comments
    ======== ===== ===========================================================
    05/19/95  1.00 Began work on this program
}
unit Sprites;

interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics;

const
  MaxFrames = 14;
  
type
TSprite = class
  Width     : Integer;
  Height    : Integer;
  LimWidth  : Integer;
  LimHeight : Integer;
  x         : Integer;
  y         : Integer;
  AnimCnt   : Integer;
  AnimSpd   : Integer;
  VelocX    : Integer;
  VelocY    : Integer;
  Canv      : TCanvas;
  Frames    : Array [0..MaxFrames] of TBitMap;
  Masks     : Array [0..MaxFrames] of TBitMap;
  BackGrnd  : TBitmap;
  OffScrn   : TBitmap;
  BakScrn   : TBitmap;
  CurFrame  : Integer;
  NumFrames : Integer;
  rcImg     : TRect;
  rcBack    : TRect;
  rcDirty   : TRect;

  constructor Create(Canvas: TCanvas; os, bs: TBitMap; W, H: Integer); dynamic;
  procedure   Free; dynamic;
  procedure   GetBehind;
  procedure   GetFrames(FileName: String; W, H, F, Row, Cols: Integer);
  procedure   MoveIt;
  procedure   SetAnimation(s: Integer);
  procedure   SetFrame(F: Integer);
  procedure   SetPos(nx, ny: Integer);
  procedure   SetVelocity(xv, yv: Integer);
  procedure   Translate;
end;

implementation
{==============================================================================
                                 Create
Purpose: Initialize a sprite object
==============================================================================}
constructor TSprite.Create(Canvas: TCanvas; os, bs: TBitMap; W, H: Integer);
var
 Cntr: Integer;
begin
  inherited Create;
  Canv            := Canvas;
  OffScrn         := os;
  BakScrn         := bs;
  LimWidth        := W;
  LimHeight       := H;
  Width           := 0;
  Height          := 0;
  NumFrames       := 0;
  rcImg           := Rect(0, 0, 0, 0);
  rcDirty         := Rect(0, 0, 0, 0);
  rcBack          := Rect(0, 0, 0, 0); 
  BackGrnd        := Nil;
  for Cntr := 0 to MaxFrames do
  begin
    Frames[Cntr] := Nil;
    Masks[Cntr]  := Nil;
  end;
  SetAnimation(1);
  SetPos(0, 0);
  SetVelocity(0, 0);
  SetFrame(0);
end;
{==============================================================================
                                 Free
Purpose: Free up all memory that this sprite needed
==============================================================================}
procedure TSprite.Free;
var
  Cntr: Integer;
begin

  if (BackGrnd <> Nil) then
    BackGrnd.Free;
  
  for Cntr := 0 to NumFrames do
  begin
    if (Frames[Cntr] <> Nil) then
      Frames[Cntr].free;
      
    if (Masks[Cntr] <> Nil) then
      Masks[Cntr].free;
  end;
  inherited Free;
end;
{==============================================================================
                                 GetBehind
Purpose: Save the Background From Where the Image will be drawm
Details: This is saved from the back screen due to the fact that other sprites
         might be in the way and we would capture them and not the background
         This is simple because we would want to capture a stationary sprite
         but this is for demo purposes only
==============================================================================}
procedure TSprite.GetBehind;
begin
  rcBack := Bounds(x, y, Width, Height);
  BackGrnd.Canvas.CopyRect(rcImg, BakScrn.Canvas, rcBack);
end;
{==============================================================================
                                 GetFrames
Purpose: To Load the frames that this sprite uses and create their masks
Details: All Sprites Transparent Color must be black due toi the way I create
         the mask
==============================================================================}
procedure TSprite.GetFrames(FileName: String; W, H, F, Row, Cols: Integer);
var
  AllFrames: TBitmap;
  Cntr     : Integer;
  FRect    : TRect;
  IRect    : TRect;
  CurRow   : Integer;
  CurCol   : Integer;
  clrOld   : TColor;

begin
  Width           := W;           {Width  of Each Frame for the Sprite}
  Height          := H;           {Height of Each Frame for the Sprite}
  NumFrames       := F - 1;       {Total Number of Frames This sprite wants}
  BackGrnd        := TBitmap.Create; {Used to save what is behind the sprite}
  BackGrnd.Height := H; 
  BackGrnd.Width  := W;
  rcImg           := Rect(0, 0, W, H);
  IRect           := Rect(0, 0, W, H);
  
      {Calculate the offset to the Row in the file are we starting on}
  CurRow          := Row  * (Height + 1) + 1;
  
      {Calculate the offset to the end of one column }
  Cols            := Cols * (Width  + 1);
  
  CurCol          := 1;
  if (NumFrames > MaxFrames) then
    NumFrames := MaxFrames;

  try
    AllFrames  := TBitmap.Create;
    AllFrames.LoadFromFile(FileName);
    
          { Load the frames From The File }
    for Cntr := 0 to NumFrames do   
    begin
      Frames[Cntr]        := TBitmap.Create;
      Frames[Cntr].Width  := Width;
      Frames[Cntr].Height := Height;
      FRect               := Bounds(CurCol, CurRow, Width, Height);
      Frames[Cntr].Canvas.CopyRect(IRect, AllFrames.Canvas, FRect);

             { Create A Mask for the Frame }
      Masks[Cntr]            := TBitmap.Create;
      Masks[Cntr].Width      := Width;
      Masks[Cntr].Height     := Height;
      Masks[Cntr].MonoChrome := TRUE;

             { Use BitBlts Built in ability to convert Color to mono
               By Setting the Source Frames Color To black we automatically
               are Creating a Mask When We BitBlt }
               
      clrOld := GetBkColor(Frames[Cntr].Canvas.Handle);
      SetBkColor(Frames[Cntr].Canvas.Handle, clBlack);
      BitBlt(Masks[Cntr].Canvas.Handle, 0, 0, Width, Height,
             Frames[Cntr].Canvas.Handle, 0, 0, SRCCOPY);
      SetBkColor(Frames[Cntr].Canvas.Handle, clrOld);

      CurCol := CurCol + Width + 1;
      if (CurCol >= Cols) then
      begin
        CurRow := CurRow + Height + 1;
        CurCol := 1;
      end;
    end;
  finally
    AllFrames.Free;
  end;
end;
{==============================================================================
                                 MoveIt
Purpose: To Draw and Move A Sprite
Details: Erase the Sprite from its old position by copying the background that
         Used to be behind it.  Then Move the Sprite, then draw the mask
         and then the frame at the new location, and finally Update the canvas
==============================================================================}
procedure TSprite.MoveIt;
begin
                { Erase The Image at the last location}
  rcDirty := Bounds(x, y, Width, Height);
  
  OffScrn.Canvas.CopyMode := cmSrcCopy;
  OffScrn.Canvas.CopyRect(rcDirty, BackGrnd.Canvas, rcImg);

              { Move To the New Position this sets rcBack to new location}
  Translate;

           { Display New Postion Using Mask and Actual Image }
  OffScrn.Canvas.CopyMode := cmSrcAnd;
  OffScrn.Canvas.CopyRect(rcBack, Masks[CurFrame].Canvas, rcImg);

  OffScrn.Canvas.CopyMode := cmSrcPaint;
  OffScrn.Canvas.CopyRect(rcBack, Frames[CurFrame].Canvas, rcImg);

            { Determine How Much Needs to Be Displayed on Screen
              By Drawing to the OffScreen First and then Getting the
              Rectangle From where it used to be and to where it is
              now we only Have to do 1 copy to the Forms Canvas}
  UnionRect(rcDirty, rcDirty, rcBack);
  Canv.CopyRect(rcDirty, OffScrn.Canvas, rcDirty);

            { Get the Next Frame }
  Inc(AnimCnt);
  if (AnimCnt = AnimSpd) then
  begin
    AnimCnt := 0;
    Inc(CurFrame);
    if (CurFrame > NumFrames) then
      CurFrame := 0;
   end;
end;
{==============================================================================
                                 SetAnimation
Purpose: To Set how fast the Sprite Changes its image
==============================================================================}
procedure TSprite.SetAnimation(s: Integer);
begin
  if (AnimSpd > 0) then
    AnimSpd := s
  else
    AnimSpd := 1;
  AnimCnt := 0;
end;
{==============================================================================
                                 SetFrame
Purpose: Sets the Sprites Current Frame
==============================================================================}
procedure TSprite.SetFrame(F: Integer);
begin
  CurFrame := F;
end;
{==============================================================================
                                 SetPos
Purpose: Set the lcoation of the sprite
==============================================================================}
procedure TSprite.SetPos(nx, ny: Integer);
begin
  x := nx;
  y := ny;
  if (BackGrnd <> Nil) then
    GetBehind;
end;
{==============================================================================
                                 SetVelocity
Purpose: Set how fast and what direction the sprite moves
==============================================================================}
procedure TSprite.SetVelocity(xv, yv: Integer);
begin
  VelocX := xv;
  VelocY := yv;
end;
{==============================================================================
                                 Translate
Purpose: To move the sprite and checks its bounds
==============================================================================}
procedure TSprite.Translate;
begin
  x := x + VelocX;
  y := y + VelocY;

  if (x > LimWidth) then
    x := 0
  else if (x < 0) then
    x := LimWidth;

  if (y > LimHeight) then
    y := 0
  else if (y < 0) then
    y := LimHeight;

  GetBehind;
end;

end.
