{
    Project: Drawing Sprites
    Company: Word in Action
             Copyright (C) 1995 by Jay Giganti.  All Rights Reserved.
             

    File    : sprite_s
    Author  : Jay Giganti
    Overview: This is the main form that creates the sprites,
              backgrounds and gets things going.

              If you only want to see this thing move at the clock
              resolution of 18.2 times a second then  LoopMode = 1
              in the ini file, otherwise set it to 0 for a tight loop.
              

    Disclaimer: This code is for demonstration purposes only and should
                be used at your own risk

                  Any comments, suggestions or improvements:
                  Jay Giganti -- CompuServe ID: [72037,2225]


                              Version Information
      Date    Ver  Comments
    ======== ===== ===========================================================
    05/19/95  1.00 Began work on this program
}

unit Sprite_s;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons, Sprites, IniFiles;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    Panel1: TPanel;
    Panel2: TPanel;
    bitbtn1: TButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Panel2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
const
  TightLoop = 0;
  TimerLoop = 1;
  MaxShips  = 10;
  
var
  LoopMode   : Integer;
  NumShips   : Integer;
  Ships      : Array[0..MaxShips] of TSprite;
  StartTicks : LongInt;
  TimesDrawn : LongInt;
  bDone      : Boolean;
  OffScrn    : TBitMap;
  BackScrn   : TBitMap;
  ChangeTime : Integer;
  AnimTime   : Integer;
  Veloc      : Integer;
  dx         : Integer;
  iniFile    : TIniFile;
  iniPath    : String;

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
  Cntr     : Integer;
begin
  Randomize;

  IniPath    := ExtractFilePath(Application.ExeName) + 'sprites.ini';
  iniFile    := TiniFile.Create(IniPath);
  LoopMode   := iniFile.ReadInteger('Settings', 'LoopMode',       TightLoop);
  NumShips   := iniFile.ReadInteger('Settings', 'NumShips',       3);
  ChangeTime := iniFile.ReadInteger('Settings', 'Max ChangeTime', 1000);
  AnimTime   := iniFile.ReadInteger('Settings', 'Max AnimTime',   200);
  Veloc      := iniFile.ReadInteger('Settings', 'Max Velocity',   5);
  iniFile.Free;
  
  Dec(NumShips);
  if NumShips > MaxShips then
    NumShips := MaxShips;
  
  OffScrn  := TBitmap.Create;
  BackScrn := TBitmap.Create;
  OffScrn.LoadFromFile('B07.Bmp');  { One for drawing }
  BackScrn.LoadFromFile('B07.Bmp'); { One to get a clean background }
  ClientWidth  := OffScrn.Width;
  ClientHeight := OffScrn.Height;

  Canvas.CopyRect(Rect(0, 0, OffScrn.Width, OffScrn.Height),
                  OffScrn.Canvas, Rect(0, 0, OffScrn.Width, OffScrn.Height));
  
  for Cntr := 0 to NumShips do
  begin
    Ships[Cntr] := TSprite.Create(Canvas, BackScrn, OffScrn,
                                  ClientWidth, ClientHeight);
    Ships[Cntr].GetFrames('A07.BMP', 80, 48, 12, 0, 3);
    Ships[Cntr].SetVelocity(Random(Veloc), Random(Veloc));
    Ships[Cntr].SetPos(Random(ClientWidth), Random(ClientHeight));
    Ships[Cntr].SetAnimation(Random(AnimTime));
  end;
  Panel2.Caption := 'Click To Get Timing Stats';
  dx             := Random(ChangeTime) + 1;
  TimesDrawn     := 0;
  Timer1.Enabled := TRUE;
  StartTicks     := GetTickCount;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Cntr: Integer;
begin
  for Cntr := 0 to NumShips do
    Ships[Cntr].Free;
  BackScrn.Free;
  OffScrn.Free;
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
begin
  if LoopMode = TightLoop then
    bDone := True
  else
    Close;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  Cntr,
  xv,
  yv: Integer;
begin

  if LoopMode = TightLoop then
  begin
    Timer1.Enabled := FALSE;
    bDone          := FALSE;
  end
  else
    bDone := TRUE;

  repeat
    for Cntr := 0 to NumShips do
    begin
      Ships[Cntr].MoveIt;
      Inc(TimesDrawn);
    end;
    
    if (TimesDrawn Mod dx) = 0 then
    begin
      dx       := Random(ChangeTime) + 1;
      for Cntr := 0 to NumShips do
      begin
        xv := Random(Veloc);
        yv := Random(Veloc);
        
        if (Random(2) = 1) then
          xv := -xv;
          
        if (Random(2) = 1) then
          yv := -yv;
          
        Ships[Cntr].SetVelocity(xv, yv);
        Ships[Cntr].SetAnimation(Random(AnimTime));
      end;
    end;
    
    if LoopMode = TightLoop then
      Application.ProcessMessages;
  until bDone;

  if LoopMode = TightLoop then
    Close;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  Canvas.CopyRect(Rect(0, 0, OffScrn.Width, OffScrn.Height),
                  OffScrn.Canvas, Rect(0, 0, OffScrn.Width, OffScrn.Height));
end;

procedure TForm1.Panel2Click(Sender: TObject);
var
  Seconds  : Real;
  DrawsSec : Real;
  DrawType : String;
begin
  if LoopMode = TightLoop then
    DrawType := 'Tight Loop'
  else
    DrawType := 'Timer Loop';

  Seconds        := (GetTickCount - StartTicks) / 1000;
  DrawsSec       := TimesDrawn / Seconds;
  Panel2.Caption := Format('%s- Draws: %d  Seconds: %8.4f  Draws/Sec: %8.4f',
                            [DrawType, TimesDrawn, Seconds, DrawsSec]);
                            
  iniFile := TiniFile.Create(IniPath);
  iniFile.WriteString('Timing', 'DrawType', DrawType);
  iniFile.WriteInteger('Timing', 'TimesDrawn', TimesDrawn);
  FmtStr(DrawType, '%4.4f', [Seconds]);
  iniFile.WriteString('Timing', 'Seconds', DrawType);
  FmtStr(DrawType, '%4.4f', [DrawsSec]);
  iniFile.WriteString('Timing', 'DrawsPerSecond', DrawType);
  iniFile.Free;
  
  TimesDrawn     := 0;
  StartTicks     := GetTickCount;

end;

end.

